# -*- coding: utf-8 -*-
"""
    crimson
    ~~~~~~~

    :license: BSD

    Converter for various bioinformatics tool outputs.

"""
# (c) 2015-2018 Wibowo Arindrarto <bow@bow.web.id>

RELEASE = True

__version_info__ = ("0", "4", "0")
__version__ = ".".join(__version_info__)
__version__ += "-dev" if not RELEASE else ""

__author__ = "Wibowo Arindrarto"
__contact__ = "bow@bow.web.id"
__homepage__ = "http://bow.web.id"
